set -e

while [ $# -gt 0 ]; do
    case "$1" in
	-P) OPT_PLOT_DISABLE=1; shift;;
	-p) OPT_PLOT_ONLY=1; shift;;
	-X|--no-x11-plot) OPT_NO_X11=1; shift;;
    esac
done

PATH=$PWD/../_compiled/bin/:$PATH

COUNT=10000

error() {
    echo $1 >&2
    exit 1
}

sshgw() {
    local socket="$HOME/.ssh/cangw-connection"

    if [[ ! -S $socket ]]; then
        # Create master connection to speed up subsequenct command.
        # The ssh is put into background and the connection is closed
        # after 10 minutes)
	ssh -f -M -S $socket root@192.168.2.3 sleep 600 > /dev/null 2>&1
    fi
    ssh -x -a -S $socket root@192.168.2.3 "$@"
}

_cleanupgw() {
    # Set can interfaces up
    sshgw 'for i in 0 1; do ip link show dev can$i|grep -q UP || ip link set can$i up type can bitrate 1000000; done'
    # Delete all GW rules
    sshgw 'eval $(cangw -L|sed -e s/-A/-D/ -e "s/#.*/;/")'
}

_plot() {
    local testname=`basename $0 .sh`

    cmd=$(plot_cmds)			# Get plot commands
    cmd=$(echo "$cmd" | sed -e "/set title/ s/[\"']\(.*\)[\"']/\"\1\\\\n($kvers)\"/")
    if [ -z "$OPT_NO_X11" ]; then
	echo "set terminal x11 enhanced; ${cmd}" | gnuplot -persist
    fi
    I=''
    echo 'set terminal postscript color eps enhanced size 6cm,4cm lw 1 "Times-Roman" 10;' \
	'set lmargin 8;' \
	"${cmd}" | gnuplot > ${testname}$I.eps
    echo "set terminal postscript color eps enhanced;" \
	"${cmd}" | gnuplot | epstopdf --filter > ${testname}$I.pdf
    mkdir -p thumb
    convert -density 30  -gamma 0.5 -quality 90 -type Palette -depth 8 ${testname}$I.pdf thumb/${testname}$I.png
    convert -density 150 -gamma 0.7 -quality 90 -type Palette -depth 8 ${testname}$I.pdf ${testname}$I.png

}

create_dirs_and_links() {
    local test=$1
    local kver=$2

    local  d=results/by-kern/$kver/$test
    mkdir -p $d
    mkdir -p results/by-test/$test
    ln -sfT ../../${d#results/} results/by-test/$test/$kver
    echo $d
}

_run() {
    _cleanupgw
    kvers=$(sshgw uname -r)
    dir=$(create_dirs_and_links $(basename $0 .sh) $kvers)
    cd $dir
    if [[ ! "$OPT_PLOT_ONLY" ]]; then
	main
    fi
    _plot
}

test_end() {
    test_end_called=t
    trap '' DEBUG
    _run
}

exit() {
    if [[ ! "$test_end_called" ]]; then
	test_end_called=t
	error "bug in the test script: No test_end called"
    fi
}

trap 'exit' EXIT
